' Region Description
'
' Name: Cindy
' Author: Nils Kaczenski, faq-o-matic.net
' Version: 1.5 (May 11, 2015)
' Description: Cindy reads configuration data from a Windows system
'   using WMI and writes a text-based report.
'   You can easily extend the report data by adding WMI classes and
'   properties to the code.
' Tested on: Windows Server 2012/R2, Windows 8/8.1, Windows 7, Windows Server 2008 R2, Windows Server 2008 (x64), Windows Server 2003 (x86), Windows XP
' Will not run on: Windows RT
' Updates:
'	1.5: correction to the batch report files (date function), no changes to VBS code
'	1.4: - new date function in Local Report and Remote Report batches (avoiding date format issues)
'		 - no changes to VBS code
'   1.3: - added system serial number to "basic" and "quick" modes
'        - extended Local Report batch
'	1.2: - added script mode "basic" to only report basic config data
'		 - added filters for Network Adapters and their Configuration
'		 - added Win32_ServerFeature to detect server roles in Windows Server 2008 and later
'	1.1: - added script modes "quick" (omits long-running queries) and "all" (does all queries)
'			with "quick" as default if mode is not specified
'		 - if a query runs into an error Cindy will proceed with the next query instead of terminating
'
' EndRegion

debugmode = False
dtmStart = Now
strTitle = "Cindy 1.5 by faq-o-matic.net"

' make sure we run with cscript
If InStr(LCase(WScript.FullName), "cscript") = 0 Then
	strCmd = "cscript //nologo " & WScript.ScriptFullName
	InputBox "This script must be run from a CMD window using cscript." & vbNewLine _
		& "Use the following command line." & vbNewLine _
		& "You may specify a computer name as an argument." & vbNewLine _
		& "Set the /mode:all switch to perform all documentation." & vbNewLine _
		& "Set the /mode:basic switch to only collect basic data." & vbNewLine _
		& "It is best to redirect the output to a file.", strTitle, strCmd
	WScript.Quit
End If

' find out computer and user name
On Error Resume Next
Set objWSHNetwork = CreateObject("Wscript.Network")
strComputer = objWSHNetwork.ComputerName
strUser = objWSHNetwork.UserDomain & "\" & objWSHNetwork.UserName
If checkit("Error accessing WSHNetwork!", True) Then WScript.Quit
On Error Goto 0

' get the mode passed via parameters
Set objArgsN = WScript.Arguments.Named
' name the accepted modes - this is for future extensibility
strMode = LCase(WScript.Arguments.Named("mode"))
if strMode = "" then strMode = "quick"

' if computer name was passed by the command line use this name
Set objArgsU = WScript.Arguments.Unnamed
If objArgsU.Count <> 0 Then
	' we take only the first computer name
	strComputer = objArgsU(0)
End If

' connect to WMI
On Error Resume Next
Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
If checkit("Error connecting to WMI at " & strComputer & "!", true) Then WScript.Quit
On Error Goto 0

' define the WMI classes and properties to read
'   you can extend this easily by copying one of the lines and modifying
'   the WMI class and properties
'   note: Caption will always be included automatically
Set dctWMI = CreateObject("Scripting.Dictionary")
dctWMI.Add "Win32_ComputerSystem", "AutomaticManagedPagefile,AutomaticResetBootOption,Domain,DomainRole,Manufacturer,Model,NumberOfProcessors,NumberOfLogicalProcessors,PrimaryOwnerName,TotalPhysicalMemory"
dctWMI.Add "Win32_SystemEnclosure", "SerialNumber"
dctWMI.Add "Win32_OperatingSystem", "BootDevice,CodeSet,CountryCode,CurrentTimeZone,InstallDate,OSArchitecture,OSLanguage,ServicePackMajorVersion,ServicePackMinorVersion,SystemDevice,WindowsDirectory"
dctWMI.Add "Win32_Processor", "Name,NumberOfCores,CurrentClockSpeed"
dctWMI.Add "Win32_PhysicalMemory", "BankLabel,Capacity"
dctWMI.Add "Win32_DiskDrive", "InterfaceType,Index,Size"
dctWMI.Add "Win32_DiskPartition", "Size"
dctWMI.Add "Win32_LogicalDisk", "Size,FreeSpace"
dctWMI.Add "Win32_NetworkAdapter", "AdapterType,Manufacturer,PhysicalAdapter,MACAddress,NetConnectionID"
dctWMI.Add "Win32_NetworkAdapterConfiguration", "MACAddress,IPEnabled,IPAddress,IPSubnet,DefaultIPGateway,DHCPEnabled,DNSDomain,DNSServerSearchOrder,WINSPrimaryServer,WINSSecondaryServer" 
dctWMI.Add "Win32_ServerFeature", "ID,ParentID" 
If strMode = "all" Then
	dctWMI.Add "Win32_UserAccount", "Disabled"
	dctWMI.Add "Win32_Group", "Description"
	dctWMI.Add "Win32_Share", "Name,Path"
	dctWMI.Add "Win32_Product", "InstallDate,InstallLocation,Vendor,Version"
	dctWMI.Add "Win32_Service", "Name,PathName,StartMode,StartName"
	dctWMI.Add "Win32_QuickFixEngineering", "HotFixID,Description"
End If


' Indention for sub-items
strIndent = "   "

' Display header and basic info
WScript.Echo "Configuration Report for: " & strComputer
WScript.Echo String(Len(strComputer) + 26, "=")
WScript.Echo "Generated on: " & Now & " by: " & strUser
WScript.Echo "Script mode: " & strMode
WScript.Echo vbNewLine
' Get IP Addresses for the local computer
Set colNIC = objWMIService.ExecQuery("Select * From Win32_NetworkAdapterConfiguration Where IPEnabled = True")

If strMode = "basic" Then
	' display basic data only
	GetBasicData
Else
	WScript.Echo "IP basic configuration:"
	For Each objItem In colNIC
		WScript.echo strIndent & objItem.Caption & ": " & Join(objItem.IPAddress, ", ") & "; " & objItem.MACAddress 
	Next
	WScript.Echo vbNewLine
	' this is the main routine that walks through all classes
	For Each strWMIClass in dctWMI
		GetWMIData strWMIClass, dctWMI.Item(strWMIClass)
		WScript.Echo vbNewLine
	Next
End If


WScript.Echo vbNewLine & "Collected by " & strTitle
WScript.Echo strIndent & "The report took " & DateDiff("s", dtmStart, Now) & " seconds to complete."

Function GetWMIData(strWMIClass, strWMIProperties)
	' collect WMI data for the given class and the given properties
	' and display it
	strClass = Replace(strWMIClass, "Win32_", "")
	WScript.Echo strClass
	WScript.Echo String(Len(strClass), "-")
    strWQL = "SELECT * FROM " & strWMIClass 
	' provide filters for special classes
	Select Case strWMIClass
		Case "Win32_UserAccount" strWQL = strWQL & " WHERE LocalAccount=True"
		Case "Win32_Group" strWQL = strWQL & " WHERE LocalAccount=True"
	End Select
	If strWMIClass = "Win32_NetworkAdapter" And strMode <> "all" Then strWQL = strWQL & " WHERE MacAddress IS NOT NULL"
	If strWMIClass = "Win32_NetworkAdapterConfiguration" And strMode <> "all" Then strWQL = strWQL & " WHERE IPEnabled = True"

	On Error Resume Next
	Set objClass = objWMIService.Get(strWMIClass)
    Set colItems = objWMIService.ExecQuery(strWQL, , 48)
	' quit this query if we run along an error
	If Checkit("WMI error!", True) Then Exit Function
	' Inconsistently, Win32_ServerFeature does not use the Caption property ...
	If strWMIClass = "Win32_ServerFeature" Then
		arrWMIProperties = Split("Name," & strWMIProperties, ",")
	Else
		arrWMIProperties = Split("Caption," & strWMIProperties, ",")
	End If

	intItemCount = 0
	For Each objItem In colItems
		intItemCount = intItemCount + 1
		For Each strProperty In arrWMIProperties
			' do we have an array?
			If objItem.Properties_(strProperty).IsArray Then
				' yes: join the values
				strInfoText = Join(objItem.Properties_(strProperty).Value, ", ")
			Else
				' no: just fetch the value
				strInfoText = objItem.Properties_(strProperty).Value
			End If
			' did we get an error? If so, display "no value"
			if Checkit(strProperty, False) then strInfoText = "(no value)"
			' recalculate some byte values to a more readable format
			If (strProperty = "Size" Or strProperty = "Capacity" Or strProperty = "TotalPhysicalMemory" Or strProperty = "FreeSpace") And IsNumeric(strInfoText) Then
				strInfoText = strInfoText & " Bytes (" & Round(strInfoText/1024/1024, 2) & " MB; " & Round(strInfoText/1024/1024/1024, 2) & " GB)"
			End If

			' format the output
			If strProperty = "Caption" Or (strWMIClass = "Win32_ServerFeature" And strProperty = "Name") Then
				WScript.Echo strInfoText
			Else
				WScript.Echo strIndent & strProperty & ": " & strInfoText
			End If

		Next
	Next
	If intItemCount > 1 Then WScript.Echo "-> " & intItemCount & " total items for " & strWMIClass
	If Instr(lcase(strWQL), " where ") > 0 then WScript.Echo "-> (Filter applied to only display relevant data. Use /mode:all to remove the filter; user accounts always filter for local objects.)"
	On Error Goto 0
End Function

Function GetBasicData
	' Function that collects and displays basic data about a computer system
	' First step: Collect data
	On Error Resume Next
	Set objAquarium = objWMIService.InstancesOf("Win32_ComputerSystem",48)
	For Each objFish In objAquarium
		strModel = objFish.Model
		strManufacturer = objFish.Manufacturer
		' Get Domain Role (see http://msdn.microsoft.com/en-us/library/windows/desktop/aa394102(v=vs.85).aspx)
		Select Case objFish.DomainRole
			Case 0 strDomRole = "Standalone Workstation"
			Case 1 strDomRole = "Member Workstation"
			Case 2 strDomRole = "Standalone Server"
			Case 3 strDomRole = "Member Server"
			Case 4 strDomRole = "Domain Controller"
			Case 5 strDomRole = "Domain Controller (PDC Emulator)"
		End Select
		strDomain = objFish.Domain
		intRAM = objFish.TotalPhysicalMemory
		intRAMGB =  Round(intRAM / 1024 / 1024 / 1024, 2) 
		strCPU = objFish.NumberOfProcessors
		strLogCPU = objFish.NumberOfLogicalProcessors
	Next	
	If strLogCPU <> "" Then strCPU = strCPU & " (logical CPUs: " & strLogCPU & ")"
	Set objAquarium = objWMIService.InstancesOf("Win32_SystemEnclosure",48)
	For Each objFish In objAquarium
		strSerial = objFish.SerialNumber
	Next
	Set objAquarium = objWMIService.InstancesOf("Win32_OperatingSystem",48)
	For Each objFish In objAquarium
		strOS = objFish.Caption
		strOSSP = objFish.ServicePackMajorVersion
	Next
	If strOSSP <> "" Then strOSSP = "SP " & strOSSP
	For Each objItem In colNIC
		strIPAddr = strIPAddr & Join(objItem.IPAddress, ", ") & "; "
	Next
	strIPAddr = Left(strIPAddr, Len(strIPAddr)-2)
	Set objAquarium = objWMIService.InstancesOf("Win32_DiskDrive",48)
	For Each objFish In objAquarium
		strDrive = objFish.Caption
		intDriveSize = objFish.Size
		strDrives = strDrives & strDrive & ": " & Round(intDriveSize / 1024 / 1024 / 1024, 2) & " GB; "
	Next
	strDrives = Left(strDrives, Len(strDrives)-2)
	On Error Goto 0
	' Second step: Display data
	WScript.Echo "Computer Name: " & strComputer
	WScript.Echo "System Model: " & strModel & " (" & strManufacturer & ")"
	WScript.Echo "Serial No: " & strSerial
	WScript.Echo "Operating System: " & strOS & " " & strOSSP
	WScript.Echo "Domain Role: " & strDomRole & " (" & strDomain & ")"
	WScript.Echo "IP Addresses: " & strIPAddr
	WScript.Echo "RAM: " & intRAMGB & " GB"
	WScript.Echo "CPU: " & strCPU 
	WScript.Echo "Physical Drives: " & strDrives

End Function


Function Checkit(strText, blnDisplay)
	' function to check for errors
	' error will be displayed with strText if blnDisplay is true or debugmode is true
	Checkit = False
	If Err.number <> 0 Then
		strError = strText & " (" & Err.number & ": " & Err.Description & ")"
		Err.Clear
		if debugmode or blnDisplay then WScript.Echo strError
		Checkit = True
	End If

End Function
